module Pot
  class Deployment

    class UndefinedInstance < ::StandardError
      def message
        "Instance definition not found for key: #{@message}"
      end
    end

    class UndefinedPolicy < ::StandardError
      def message
        "Policy definition not found for key: #{@message}"
      end
    end

    def initialize(options = {})
      @options = options
    end

    def execute_on_instances(commands, instance_name)
      instances = find_instances(instance_names)
      deploy_to_instances(instances) do |actor|
        actor.execute(commands)
      end
    end

    def deploy_policies_to_instances(police_names, instance_names)
      policies = find_policies(police_names)
      instances = find_instances(instance_names)
      deploy_policies_to_instances_internal(policies, instances)
    end

    def deploy_packages_to_instances(package_names, instance_names)
      package_tree = Pot::Bundle.new(package_names)
      instances = find_instances(instance_names)
      deploy_to_instances(instances) do |actor|
        package_tree.list.each do |package|
          package.process(actor)
        end
      end
    end

    def deploy_roles(roles)
      roles = roles.map(&:to_sym)
      instances = []
      Pot::Role::REGISTER.each do |role_name, instance|
        instances << instance if roles.include?(role_name)
      end
      policies = []
      Pot::Policy::REGISTER.each do |policy_name, policy|
        policy_roles = [policy.roles].flatten
        policies << policy unless (policy_roles & roles).empty?
      end
      deploy_policies_to_instances_internal(policies, instances)
    end

    private

      def find_policies(police_names)
        [police_names].flatten.map do |policy_name|
          policy = Pot::Policy::REGISTER[policy_name.to_sym]
          raise UndefinedPolicy, policy_name unless policy
          policy
        end
      end

      def find_instances(instance_names)
        [instance_names].flatten.map do |instance_name|
          instance = Pot::Instance::REGISTER[instance_name.to_sym]
          raise UndefinedInstance, instance_name unless instance
          instance
        end
      end

      def deploy_policies_to_instances_internal(policies, instances)
        deploy_to_instances(instances) do |actor|
          policies.each do |policy|
            policy.process(actor)
          end
        end
      end

      def deploy_to_instances(instances)
        instances.each do |instance|
          actor = Pot::Actor.new(instance, @options)
          yield actor
        end
      end

  end
end
