module Pot
  # = Policies
  #
  # A policy defines a set of packages which are required for a certain
  # role (app, database, etc.). All policies defined will be run and all
  # packages required by the policy will be installed. So whereas defining
  # a Pot::Package merely defines it, defining a Pot::Policy
  # actually causes those packages to install.
  #
  # == A Basic Example
  #
  #   policy :blog, :roles => :app do
  #     require :webserver
  #     require :database
  #     require :rails
  #   end
  #
  # This says that for the blog on the app role, it requires certain
  # packages. The :roles option is <em>exactly</em> the same as a capistrano
  # or vlad role. A role merely defines what server the commands are run
  # on. This way, a single Pot script can provision an entire group
  # of servers.
  #
  # To define a role, put in your actor specific configuration file (recipe or
  # script file):
  #
  #   role :app, "208.28.38.44"
  #
  # The capistrano and vlad syntax is the same for that. If you're using a
  # custom actor, you may have to do it differently.
  #
  # == Multiple Policies
  #
  # You may specify as many policies as you'd like. If the packages you're
  # requiring are properly defined with verification blocks, then
  # no software will be installed twice, so you may require a webserver on
  # multiple packages within the same role without having to wait for
  # that package to install repeatedly.
  class Policy #:nodoc:
    REGISTER = {}

    attr_reader :name,
                :packages,
                :roles

    def initialize(name, metadata = {}, &block)
      raise 'No name provided' unless name
      raise 'No roles provided' unless metadata[:roles]

      @name = name.to_sym
      @roles = metadata[:roles]
      @package_tree = Pot::Bundle.new

      REGISTER[@name] = self

      self.instance_eval(&block)
    end

    def requires(package, options = {})
      @package_tree.add(package)
    end

    def to_s
      name
    end

    def process(actor)
      @package_tree.list do |package|
        package.process(actor)
      end
    end

  end
end
